!*************************************************************************************************
! Climate statistics recorder
!*************************************************************************************************
SUBROUTINE climate_stat &
   (tmp_air, tmp_soil, prec, par_grass,                   &
   doy, year, counter, phenology, pheno_clo, pheno_cls,   &
   C3g_no, C4g_no,                                        &
   tmp_air_RunningRecord,   prec_RunningRecord,           &
   tmp_soil_RunningRecord,  tmp_soil_month_ave,           &
   tmp_coldest_RunningRecord, tmp_coldest_20yr_ave,       &
   tmp_hottest_RunningRecord, tmp_hottest_20yr_ave,       &
   tmp_ave_C3growth_RunningRecord, tmp_recorder_C3growth, &
   par_grass_RunningRecord                                )

USE data_structure
implicit none

!Define variables
real   ,intent(IN)::tmp_air   !2m air temperature (Celcius)
real   ,intent(IN)::tmp_soil  !soil temperature (Celcius)
real   ,intent(IN)::prec      !precipitation (mm day-1)
real   ,intent(IN)::par_grass !total PAR on top of the grass layer (micro mol photon m-2 s-1)

integer,intent(IN)::doy       !day of the year (1-365)
integer,intent(IN)::year      !year of simulation
integer,intent(IN)::counter   !simulation day (1-Simulation_year*365)

integer,intent(IN)::C3g_no    !PFT number of C3 grass
integer,intent(IN)::C4g_no    !PFT number of C4 grass

logical,dimension(1:PFT_no),intent(IN)::phenology !Phenological status (T:foliation, F:dormance)
integer,dimension(1:PFT_no),intent(IN)::pheno_clo !counter number of last leaf onset
integer,dimension(1:PFT_no),intent(IN)::pheno_cls !counter number of last leaf shedding

real,dimension(365)::tmp_air_RunningRecord    !365days running record  of tmp_air (Celcius)
real,dimension(365)::prec_RunningRecord       !365days running record  of prec (mm/year)

real,dimension(30)::tmp_soil_RunningRecord    !30days running record  of tmp_soil (Celcius)
real                tmp_soil_month_ave        !30days running average of tmp_soil (Celcius)

real,dimension(20)::tmp_coldest_RunningRecord !20yr running record  of tmp_coldest_month (Celcius)
real                tmp_coldest_20yr_ave      !20yr running average of tmp_coldest_month (Celcius)

real,dimension(20)::tmp_hottest_RunningRecord !20yr running record  of tmp_hottest_month (Celcius)
real                tmp_hottest_20yr_ave      !20yr running average of tmp_hottest_month (Celcius)

real,dimension(365)::par_grass_RunningRecord  !365 days running record of par_grass

real,dimension( 20)::tmp_ave_C3growth_RunningRecord
                 !20yr running record of average temperature of growth phase of C3 grass (Celcius)
real,dimension(  2)::tmp_recorder_C3growth
                 !temperature recorder for C3 growht phase of C3 grass

!Define local variables
real tmp_air_month_ave      !monthly average of air temperature (Celcius)

!temperature recorder for C3 growht phase of C3 grass
   !on growth phase
   if ( phenology(C3g_no) ) then
      tmp_recorder_C3growth(1) = tmp_recorder_C3growth(1) + 1
      tmp_recorder_C3growth(2) = tmp_recorder_C3growth(2) + tmp_air
   endif
   
   !on the day of last phenology day
   if ( counter == pheno_cls(C3g_no) ) then
      tmp_ave_C3growth_RunningRecord(2:20) = tmp_ave_C3growth_RunningRecord(1:19)
      tmp_ave_C3growth_RunningRecord(1)    = tmp_recorder_C3growth(2) / &
                                             max(tmp_recorder_C3growth(1), 1.0)
      tmp_recorder_C3growth(:) = 0.0
   endif
   
!air temperature
   tmp_air_RunningRecord(2:365) = tmp_air_RunningRecord(1:364)
   tmp_air_RunningRecord(1)     = tmp_air
   
!precipitation
   prec_RunningRecord(2:365) = prec_RunningRecord(1:364)
   prec_RunningRecord(1)     = prec
   
!soil temperature
   tmp_soil_RunningRecord(2:30) = tmp_soil_RunningRecord(1:29)
   tmp_soil_RunningRecord(1)    = tmp_soil
   tmp_soil_month_ave           = sum(tmp_soil_RunningRecord(1:30)) /30.0
   
!coldest and hottest month of the year
   !update coldest and hottest month of the year
   if (Day_of_Month(doy) == Day_in_month(Month(doy))) then	!on the end of month
       tmp_air_month_ave = &
       sum( tmp_air_RunningRecord(1:Day_in_month(Month(doy))) ) / real( Day_in_month(Month(doy)) )
       
       if (Month(doy) == 1) then	!initialize on 31 January
           tmp_coldest_RunningRecord(2:20) = tmp_coldest_RunningRecord(1:19)
           tmp_coldest_RunningRecord(1)    = tmp_air_month_ave
           tmp_hottest_RunningRecord(2:20) = tmp_hottest_RunningRecord(1:19)
           tmp_hottest_RunningRecord(1)    = tmp_air_month_ave
       else
           tmp_coldest_RunningRecord(1) = min(tmp_coldest_RunningRecord(1), tmp_air_month_ave)
           tmp_hottest_RunningRecord(1) = max(tmp_hottest_RunningRecord(1), tmp_air_month_ave)
       end if
   end if
   
   !update 20 year running mean
   if (doy == 365) then	!on the end of year
       tmp_coldest_20yr_ave = sum( tmp_coldest_RunningRecord (1:min(20,year)) ) / min(20, year)
       tmp_hottest_20yr_ave = sum( tmp_hottest_RunningRecord (1:min(20,year)) ) / min(20, year)
   end if
   
   !running record of par_grass
   par_grass_RunningRecord(doy) = par_grass
   
END SUBROUTINE climate_stat



!**************************************************************************************************
! Search existing Plant Functiona Type in the grid
!**************************************************************************************************
SUBROUTINE pft_present (pft_exist, tree_exist, pft, C3g_no, C4g_no)

USE data_structure
implicit none

!Define arguments
logical,dimension(1:PFT_no),intent(OUT)::pft_exist  !PFT exist or not (.TRUE. or .FALSE.)
logical,dimension(1:Max_no),intent(IN) ::tree_exist !tree exist or not (.TRUE. or .FALSE.) 
integer,dimension(1:Max_no),intent(IN) ::pft        !Plant Functional Type (1-PFT_no)
integer                    ,intent(IN) ::C3g_no     !PFT number of C3 grass
integer                    ,intent(IN) ::C4g_no     !PFT number of C4 grass

!Define local variables
integer	p, no	!loop counter

!Reset varialbes
pft_exist(:)      = .false.
pft_exist(C3g_no) = .true. 
pft_exist(C4g_no) = .true. 

!Servey for existance of woody PFTs
do no=1, Max_no
if (tree_exist(no)) then
    p            = pft(no)
	pft_exist(p) = .true. 
end if
end do

END SUBROUTINE pft_present



!*************************************************************************************************
! Biome detemination routine (Must be called @ doy=365)
! -- original criteria was taken from BIOME3 of A. Haxeltine & I.C. Prentice (1996) --
!*************************************************************************************************
SUBROUTINE biome_determine &
   (biome, C3g_no, C4g_no, W_fi, W_wilt, gdd0, gdd5, lai_max_wood, lai_max_grass, &
   npp_annual, pool_w1_RunningRecord)

USE data_structure
implicit none

!auguments
integer,intent(OUT)::biome         !biome type number
integer,intent(IN) ::C3g_no        !PFT number of C3 grass
integer,intent(IN) ::C4g_no        !PFT number of C4 grass
real   ,intent(IN) ::W_fi          !filed capacity   (m3/m3, 0.0 -> 1.0)
real   ,intent(IN) ::W_wilt        !wilting point    (m3/m3, 0.0 -> 1.0)
real   ,intent(IN) ::gdd0          !growing-degree-day-sum of 0 Celcius base
real   ,intent(IN) ::gdd5          !growing-degree-day-sum of 5 Celcius base
real   ,intent(IN) ::lai_max_wood  !Wood  LAI @ annual maximum  (m2/m2)
real   ,intent(IN) ::lai_max_grass !Grass LAI @ annual maximum  (m2/m2)
real,dimension(1:PFT_no),intent(IN)::npp_annual !NPP for each pft (g dm / stand / year)
real,dimension(365)     ,intent(IN)::pool_w1_RunningRecord  !1yr running record of pool_w1

!local variables
real,dimension(12)::sm     !soil moisture index for each month
real              ::sm_min !minimum value of sm
real              ::sm_ave !average value of sm

integer dominant1, dominant2 !
real    lai_max              !annual maximum of LAI (wood + grass)
integer i,j,k                !loop counter for temporal usage
real    x, y                 !for general usage

!----------- preparation -----------
   !calculate LAI max
   lai_max = lai_max_wood + lai_max_grass
   
   !determine first dominant PFT
   dominant1 = 1   !default
   x         = 0.0 
   do i = 1, PFT_no
      if (npp_annual(i) > x ) then
         x         = npp_annual(i)
         dominant1 = i
      endif
   enddo
   
   !determine secound dominant PFT
   dominant2 = 1   !default
   x         = 0.0 
   do i = 1, PFT_no
      if ( i .ne. dominant1 ) then
      if (npp_annual(i) > x ) then
         x         = npp_annual(i)
         dominant2 = i
      endif
      endif
   enddo
   
   !sm(:): soil moisture index of upper-soil-layer for each month
   do i = 1, 12                   !i: month
      k = sum(Day_in_month(1:i))  !k: ending doy of month i
      j = k - Day_in_month(i) + 1 !j: starting doy of month i
      x = sum(pool_w1_RunningRecord(j:k)) / real(k-j+1)
      sm(i) = (x/Depth1-W_wilt) / (W_fi-W_wilt)
      sm(i) = min( 1.0, max(0.0, sm(i)) )
   enddo
   sm_min = min( sm(1),sm(2),sm(3),sm(4),sm(5),sm(6),sm(7),sm(8),sm(9),sm(10),sm(11),sm(12) )
   sm_ave = sum( sm(1:12) ) / 12.0
   
!----------- Group 1 -----------
   !1: Polar desert
   if (gdd0 < 150) then
      biome = 1; return
   endif
   
!----------- Group 2 -----------
   !2: Arctic/Alpine-tundra
   if (gdd5 < 350) then
      biome = 2; return
   endif
   
!----------- Group 3 -----------
SELECT CASE (dominant1)
   !3: tropical rain forest
   !4: tropical seasonal forest
   CASE (1)
      if (lai_max>=2.5) then
         if (sm_min >= 0.50) then
            biome = 3; return
         else
            biome = 4; return
         endif
      endif
   
   !5: tropical deciduous forest
   CASE (2)
      if (lai_max>=2.5) then
         biome = 5; return
      endif
   
   !6: temperate conifer forest
   CASE (3)
      if (lai_max>=1.5) then
         biome = 6; return
      endif
   
   !7: temperate broad-leaved evergreen forest
   CASE (4)
      if (lai_max>=3.0) then
         biome = 7; return
      endif
   
   !8: temperate deciduous forest
   CASE (5)
      if (lai_max>=2.5 ) then
         biome = 8; return
      endif
   
   !9: boreal evergreen forest
   CASE (6)
         biome = 9; return
   
   !10: boreal deciduous forest
   CASE (7:8)
         biome = 10; return
   
   !11: short grass land
   !12: tall grass land
   CASE (9)
      if (lai_max_grass>1.5 .and. sm_ave<0.65) then
         if ( lai_max_grass<3.0 ) then
            biome = 11; return
         else
            biome = 12; return
         endif
      endif
   
   CASE (10)
      x = npp_annual(C3g_no) + npp_annual(C4g_no) !annual grass NPP
      y = max( 1.0 , sum(npp_annual(:))-x )     !annual woody NPP
      x = x / y                                 !annual grass-NPP / woody-NPP
      
      if (lai_max_grass>0.4 .and. x>1.8) then
         if ( lai_max_grass<3.0 ) then
            biome = 11; return
         else
            biome = 12; return
         endif
      endif
   
END SELECT

!----------- Group 4 -----------
   !13: moist savannas
   if ( (dominant1==C3g_no .or. dominant1==C4g_no) .and. lai_max>=1.5 )then
      biome = 13; return
   endif
   
   !14: dry savannas
   if ( (dominant1==C3g_no .or. dominant1==C4g_no) .and. lai_max>=0.6 )then
      biome = 14; return
   endif
   
   !15: xeric woodland / scrub
   if ( dominant1==1 .or. dominant1==2 .or. dominant1==4) then
   if ( lai_max>=1.0 ) then
      biome = 15; return
   endif
   endif
   
   if ( dominant1==3 .or. dominant1==4 .or. dominant1==6 .or. dominant1==7 .or. dominant1==8) then
   if ( lai_max>=1.5 ) then
      biome = 15; return
   endif
   endif
   
!----------- Group 5 -----------
   !16: Arid shrubland / steppe
   !17: Desert
   if ( lai_max>=0.2 ) then
      biome = 16
   else
      biome = 17
   endif
   
END SUBROUTINE biome_determine



!*************************************************************************************************
! Exchange variables
!*************************************************************************************************
!Real variables
   SUBROUTINE exchange_r(a,b)
   real    a, b !input variables to exchange
   real    x    !for temporal usage
   x = a
   a = b
   b = x
   END SUBROUTINE
   
!Integer variables
   SUBROUTINE exchange_i(a,b)
   integer    a, b !input variables to exchange
   integer    i    !for temporal usage
   i = a
   a = b
   b = i
   END SUBROUTINE
   
!Logical variables
   SUBROUTINE exchange_l(a,b)
   logical    a, b !input variables to exchange
   logical    l    !for temporal usage
   l = a
   a = b
   b = l
   END SUBROUTINE
