!**************************************************************************************************
! Calculation of photosynthesis parameters (for each PFT)
!**************************************************************************************************
SUBROUTINE photosynthesis_condition &
   (par, par_grass, par_diffuse, par_direct, &
   vpd, CO2, sl_hgt, stat_water, tmp_air, &
   tmp_air_RunningRecord, prec_RunningRecord, tmp_ave_C3growth_RunningRecord, &
   eK, lue, co2cmp, psat, c3_no, c4_no)

USE data_structure
implicit none
common PN_s, PN_r

!PFT specific constants
real,dimension(PFT_no)::PN_s !proportion of nitrogen:stem (gN / gBiomass)
real,dimension(PFT_no)::PN_r !proportion of nitrogen:root (gN / gBiomass)

!Radiation rekated 
real,intent(IN)::par         !PAR of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_grass   !PAR under tree canopy (micro mol photon m-2 s-1)
real,intent(IN)::par_diffuse !PAR of mid-day (micro mol photon m-2 s-1)
real,intent(IN)::par_direct  !PAR of mid-day (micro mol photon m-2 s-1)

!Define augments
real                  ,intent(IN)::vpd          !vapour pressure  deficit (hPa)
real                  ,intent(IN)::CO2          !ambient (canopy) CO2 concentration (ppmv)
real                  ,intent(IN)::sl_hgt       !solar hight at midday (degree)
real,dimension(PFT_no),intent(IN)::stat_water   !state of water satisfactory (0.0-1.0)
real                  ,intent(IN)::tmp_air
real,dimension(365)   ,intent(IN)::tmp_air_RunningRecord !365days running record  of tmp_air (C)
real,dimension(365)   ,intent(IN)::prec_RunningRecord    !365days running record  of prec (mm/day)
real,dimension(20)    ,intent(IN)::tmp_ave_C3growth_RunningRecord
                 !20yr running record of average temperature of growth phase of C3 grass (Celcius)

integer,intent(IN)::c3_no !PFT number of C3 grass
integer,intent(IN)::c4_no !PFT number of C4 grass

real,dimension(PFT_no),intent(OUT)::eK     !light attenuation coefficient, no dimension
real,dimension(PFT_no),intent(OUT)::lue    !light use efficiency (mol CO2 mol photon-1)
real,dimension(PFT_no),intent(OUT)::co2cmp !CO2 compensation point (ppmv)
real,dimension(PFT_no),intent(OUT)::psat   !light-saturated phtosynthesis rate(micro mol CO2 m-2 s-1)


!Define local variables
real,dimension(PFT_no)::gs_top
                       !midday stomatal conductance on top of the leaf layer (mol H2O m-2 s-1)
real   co2cell  !monthly intercellular CO2 concentration (ppmv)
real   ptop     !canopy-top photosynthetic rate
real   topt     !adjusted optimum  temperature (deg C)
real   ce_tmp   !temperature effect on photosynthesis
real   ce_co2   !stomatal effect on photosynthesis via intercellular CO2 concentration
real   ce_water !non-stomatal effect on photosynthesis

!Define local variables
real   :: a1, a2
real   :: par_top
integer:: ipt, pft, loop

Do pft = 1, PFT_no
    
    !set photosynthesis type (1 for C3 or C4)
                    ipt=0 !C3 wood
    if (pft==c3_no) ipt=3 !C3 grass
    if (pft==c4_no) ipt=4 !C4 grass
    
    !eK: adjusted light-attenuation-coefficient
    eK(pft) = EK0(pft) / min(1.0, max(0.3, sin(sl_hgt * DtoR)))
    
!loop for parameter convergence
psat(pft) = 1.0      !initial psat, initial light_saturated photosynthesis rate
co2cell  = CO2 * 0.7 !initial intercellular CO2 concentration

do loop = 0, 5
    !------------------------------------------------------------!
    !lue: light use efficiency (mol CO2 mon photon-1)
    !------------------------------------------------------------!
    if (ipt == 0 .or. ipt == 3) then
       a1 = (52.0-tmp_air) / (3.5+0.75*(52.0-tmp_air)) !temperature dependence factor
       a2 = co2cell / (90.0+0.6*co2cell)               !CO2 dependence factor
    else
       a1 = 1.0                                        !temperature insensitive
       a2 = 1.0                                        !CO2 insensitive 
    endif
    lue(pft) = Lue0(pft) * a1 * a2
    
    !------------------------------------------------------------!
    ! ce_tmp: temperature effect on photosynthesis
    !------------------------------------------------------------!
    ! optimum photosynthesis temperature
    if (ipt == 0 .or. ipt == 3) then
       topt = Topt0(pft) + 0.01 * co2cell
    else
       topt = Topt0(pft)
    end if
    
    if (pft==c3_no) then !for C3 grass
       topt = sum( tmp_ave_C3growth_RunningRecord(1:20) ) / 20.0
       topt = max(10.0, topt)
       topt = min(30.0, topt)
    endif
    
    ce_tmp = (tmp_air-Tmax(pft)) * (tmp_air-Tmin(pft)) &
             / ((tmp_air-Tmax(pft)) * (tmp_air-Tmin(pft)) - (tmp_air-topt)**2 )
    ce_tmp = min(1.0, max(0.0, ce_tmp ))    
    
    !------------------------------------------------------------!
    ! ce_co2: stomatal limitation on photosynthesis via co2cell
    !------------------------------------------------------------!
    ! compcd: CO2 compensation point
    if(ipt == 0 .or. ipt == 3) then
       a1           = 1.00+0.0451*(tmp_air-20.0)+0.000347*((tmp_air-20.0)**2)
       co2cmp(pft) = CO2cmp0(pft) * max(0.0, a1) 
       ce_co2       = 0.30 + 0.70 * (co2cell-co2cmp(pft))/(KM(pft)+co2cell)
    else
       co2cmp(pft) = CO2cmp0(pft)
       ce_co2       = 0.50 + 0.50 * (co2cell-co2cmp(pft))/(KM(pft)+co2cell)
    endif
    
    ce_co2 = min(1.0, max(0.0, ce_co2))
    
    !------------------------------------------------------------!
    ! ce_water: limitation on photosynthesis via soil water
    !------------------------------------------------------------!
    if (ipt == 4) then
      ce_water = stat_water(pft) ** Kwater(pft)
    else
      ce_water = stat_water(pft) ** Kwater(pft)
    end if
    ce_water = min(1.0, max(0.0, ce_water))
    
    !Koppen (1936) (optional)
    a1 = sum(prec_RunningRecord(1:365))            !annual precipitation (mm/year)
    a2 = sum(tmp_air_RunningRecord(1:365)) / 365.0 !annual mean temperature (Celcius)
    if ( a1 < max(100.0, 10.0*a2) ) then
       ce_water = 0.0
    endif
    
    !------------------------------------------------------------!
    !psat: light-saturated photosynthesis rate (micro mol CO2 m-2 s-1)
    !------------------------------------------------------------!
    psat(pft)= Pmax(pft) * ce_tmp * ce_co2 * ce_water
    
    !------------------------------------------------------------!
    !ptop: canopy-top photosynthetic rate (micro mol CO2 m-2 s-1)
    !------------------------------------------------------------!
    if (pft==c3_no .or. pft==c4_no) then
       par_top = par_grass !grass PFTs
    else
       par_top = par       !woody PFTs
    end if
    
    a1 = psat(pft) * lue(pft) * par_top
    a2 = psat(pft) + lue(pft) * par_top
    
    if (a2 > 0.0) then; ptop = a1/a2
    else              ; ptop = 0.0  
    endif                           
    
    !------------------------------------------------------------!
    !gs_top: stomatal conductance on top of the leaf layer (mol H2O m-2 s-1)
    !        Leuning (1995)
    !------------------------------------------------------------!
    a1 = ( GS_b2(pft)*ptop ) / ( (CO2-co2cmp(pft))*(1.0+vpd/GS_b3(pft)) )
    
    if(psat(pft)>0.0) then
       gs_top(pft) = GS_b1(pft) + max(0.0, a1)
    else
       gs_top(pft) = GS_b1(pft)
    endif
    
    !------------------------------------------------------------!
    !co2cell: intercellular CO2 concentration (ppmv)
    !------------------------------------------------------------!
    a1       = (gs_top(pft))/1.56 !1.56: conversion from H2O to CO2 conductance
    co2cell = CO2-ptop/a1    !Leuning (1995)
    co2cell = max(0.0, min(CO2, co2cell))
    
end do

END DO

END SUBROUTINE photosynthesis_condition



!**************************************************************************************************
! Daily GPP and Canopy_conductance
!**************************************************************************************************
SUBROUTINE photosynthesis &
          (par_direct, par_diffuse, par_grass, par_diffuse_rel, par_direct_rel, &
          doy, c_uptake, dlen, &
          pft_exist, phenology, eK, lue, psat, gpp, npp, &
          tree_exist, pft, height, bole, crown_area, la, mass_leaf, &
          mass_available, mort_regu1, stat_leaf_crownbottom, stat_leaf_crowntop, stat_leaf, &
          type_grass, lai_grass, gmass_available, &
          co2cmp, co2atm, vpd, canopy_cond, c3_no, c4_no)

USE data_structure
implicit none

!Define Arguments
real par_direct  !direct PAR of mid-day (micro mol photon m-2 s-1)
real par_diffuse !difussed PAR of mid-day (micro mol photon m-2 s-1)
real par_grass   !total PAR on top of the grass layer (micro mol photon m-2 s-1)

real,dimension(1:Max_hgt)          ,intent(IN)::par_diffuse_rel
  !Relative intensity of diffused PAR for each forest layer (0.0-1.0)

real,dimension(1:Max_no, 1:Max_hgt),intent(IN)::par_direct_rel
  !Relative intensity of direct PAR for each foliage layer (0.0-1.0)

integer,intent(IN)   ::doy      !day of the year (1-365)
real   ,intent(INOUT)::c_uptake !monthly carbon uptake (C g / month stand)
real   ,intent(IN)   ::dlen     !day length (hour)

logical,dimension(PFT_no),intent(IN) ::pft_exist !PFT exist or not (.TRUE. or .FALSE.)
logical,dimension(PFT_no),intent(IN) ::phenology !phenological status (T:foliation, F:dormance)
real   ,dimension(PFT_no),intent(IN) ::eK        !light attenuation coefficient, no dimension
real   ,dimension(PFT_no),intent(IN) ::lue       !light use efficiency (mol CO2 mol photon-1)
real   ,dimension(PFT_no),intent(IN) ::psat      !light-saturated rate  (micro mol CO2 m-2 s-1)
real   ,dimension(PFT_no),intent(OUT)::gpp       !GPP for each PFT (g dm / day / stand)
real   ,dimension(PFT_no),intent(OUT)::npp       !NPP for each PFT (g dm / day / stand)

logical,dimension(1:Max_no)   ,intent(IN)   ::tree_exist !tree exist or not
integer,dimension(1:Max_no)   ,intent(IN)   ::pft        !plant functional type
integer,dimension(1:Max_no)   ,intent(IN)   ::height     !tree height from 1.3m above ground (STEP)
integer,dimension(1:Max_no)   ,intent(IN)   ::bole       !bole length from 1.3m above ground (STEP)
real   ,dimension(1:Max_no)   ,intent(IN)   ::crown_area     !crown area (m2)
real   ,dimension(1:Max_no)   ,intent(IN)   ::la             !leaf area (m2)
real   ,dimension(1:Max_no)   ,intent(IN)   ::mass_leaf      !biomass of foliage (g)
real   ,dimension(1:Max_no)   ,intent(INOUT)::mass_available !biomass available (g)
real   ,dimension(1:Max_no)   ,intent(INOUT)::mort_regu1     !annual NPP (g / individual)
real   ,dimension(1:Max_no,10),intent(INOUT)::stat_leaf_crownbottom !annual NPP of foliage bottom
real   ,dimension(1:Max_no   ),intent(INOUT)::stat_leaf_crowntop    !annual NPP of foliage top
real   ,dimension(1:Max_no)   ,intent(INOUT)::stat_leaf !benefit of producing new leaf mass (g/g Leaf)

integer               ,intent(IN)   ::type_grass      !priority grass type (3:C3, 4:C4)
real   ,dimension(1:2),intent(IN)   ::lai_grass       !lai of grass (m2/m2)
real   ,dimension(1:2),intent(INOUT)::gmass_available !grass available biomass (g/m2)

real,dimension(PFT_no),intent(IN) ::co2cmp      !CO2 compensation point (ppmv)
real                  ,intent(IN) ::co2atm      !ambient (canopy) CO2 concentration (ppmv)
real                  ,intent(IN) ::vpd         !vapour pressure deficit (hPa)
real                  ,intent(OUT)::canopy_cond !stomatal conductance   (mol H2O m-2 s-1)

integer,intent(IN)::c3_no !PFT number of C3 grass
integer,intent(IN)::c4_no !PFT number of C4 grass

!Define local variables
real   ,dimension(PFT_no)::const0, const1, const2    !constant for each woody PFTs
integer,dimension(PFT_no)::save_value
real    const3             
real    gpp_ind            !GPP of each individual wood
real    lai                !leaf area index (m2/m2)
real    a0, a1, a2, a3, a4 !for general usage
real    tmp1, tmp2         !for general usage
integer i, j, no, p, gn    !for loop counter
integer count              !counter
real    unit_converter     !photosynthesis unit converter

!_____________ Set unit_converter value
   ! constant for unit conversion
   ! from (micro mol CO2 m-2 s-1) to (g dm m-2 hour-1)
   ! micro mol  -> mol       :* 10^-6
   ! mol        -> g(Carbon) :* 12
   ! secound^-1 -> hour^-1   :* 3600
   unit_converter = 3600.0 * 12.0 / 1000000.0 / C_in_drymass

!_____________ Reset variables
gpp(:)      = 0.0
npp(:)      = 0.0
canopy_cond = 0.0

!_____________ Preparation PFT specific valiables
const0(:) = 0.0
const1(:) = 0.0
const2(:) = 0.0

DO p = 1, PFT_no
if ( psat(p)>0.000 ) then
   const0(p) = GS_b2(p) * psat(p) / max(0.01, co2atm-co2cmp(p)) / (1.0 + vpd/GS_b3(p))
   const1(p) = unit_converter * 2.0 * dlen * psat(p) / eK(p)
   const2(p) = eK(p) * lue(p) / psat(p)
endif
END DO

!_____________ Woody PFTs
count = 0
DO no=1, Max_no

p = pft(no)
if ( .not. tree_exist(no) ) cycle
if ( .not. phenology(p)   ) cycle
if ( la(no)  <=0.0        ) cycle
if ( psat(p) <= 0.0       ) cycle
   
   !initialize gpp of a tree
   gpp_ind = 0.000
   
!++++++
!   lai = la(no)/real(height(no)-bole(no))/crown_area(no) !lai for a crown layer (m2/step)
!   do i = height(no), bole(no)+1, -1
!      
!      a1      = par_diffuse*par_diffuse_rel(i) + & 
!                par_direct*par_direct_rel(no,i)    !total PAR incoming to the layer
!      a2      = 1.0 + sqrt( 1.0 + a1 * const2(p) )
!      a3      = 1.0 + sqrt( 1.0 + a1 * const2(p) * exp(-1.0*eK(p)*lai) )
!      const3  = log(a2/a3)
!      a4      = const1(p) * const3 * crown_area(no) !GPP (g dm/layer/day)
!      gpp_ind = gpp_ind + a4
!      
!      !canopy conductance
!      canopy_cond = canopy_cond + crown_area(no)*( GS_b1(p)*lai + const0(p)*(2.0/eK(p))*const3 )
!++++++

!***
   tmp1 = unit_converter * dlen * psat(p)   !
   tmp2 = la(no) / (height(no)-bole(no))    !leaf area per crown layer (m2/step)
   do i = height(no), bole(no)+1, -1
      !Photosynthesis
      a1      = par_diffuse*par_diffuse_rel(i) + & 
                par_direct*par_direct_rel(no,i)    !total PAR incoming to the layer
      a2      = 1.0 - (1.0+a1*lue(p)/psat(p))**(-0.5)
      a4      = tmp1*tmp2*a2 !GPP (g dm/layer/day)
      gpp_ind = gpp_ind + tmp1*tmp2*a2
      !Canopy conductance
      canopy_cond = canopy_cond + tmp2 * ( GS_b1(p) + const0(p) * a2 )
      
      !NPP recorder for each foliage top
      if (i-bole(no) <= 10) stat_leaf_crownbottom(no, i-bole(no)) = &
                            stat_leaf_crownbottom(no, i-bole(no)) + a4
      if (i==height(no))    stat_leaf_crowntop(no) = stat_leaf_crowntop(no) + a4
   end do
   
   !Increase mass_available & c_uptake
   mass_available (no) = mass_available(no) + gpp_ind
   gpp(p)              = gpp(p)             + gpp_ind
   npp(p)              = npp(p)             + gpp_ind
   mort_regu1(no)      = mort_regu1(no)     + gpp_ind
   c_uptake            = c_uptake           + gpp_ind * C_in_drymass
   
   !Benefit of producing new leaf mass (g / g Leaf)
   stat_leaf(no) = gpp_ind / mass_leaf(no)
   
END DO

!_____________ Herbaceous PFTs
DO gn=1,2
   !set PFT number, p
                p = c4_no
   if ( gn==1 ) p = c3_no
   
   !obtain coverage area: a0
                                    a0 = 0.1 * Max_loc*Max_loc
   if ( gn==1 .and. type_grass==3 ) a0 = 0.9 * Max_loc*Max_loc
   if ( gn==2 .and. type_grass==4 ) a0 = 0.9 * Max_loc*Max_loc
   
   !obtain GPP per unit area
   if (psat(p)>0.0) then
      !obtain daily gpp per unit area (g dm / m2 / day)
      a2     = 1.0 + sqrt( 1.0 + par_grass * const2(p))
      a3     = 1.0 + sqrt( 1.0 + par_grass * const2(p) * exp(-1.0*eK(p)*lai_grass(gn)) )
      const3 = log(a2/a3)
      gpp(p) = const1(p) * const3
      
      !increase gmass_available (g dm / m2 / day)
      gmass_available(gn) = gmass_available(gn) + gpp(p)
      
      !carbon balance
      gpp(p)   = gpp(p) * a0
      npp(p)   = gpp(p)
      c_uptake = c_uptake + gpp(p) * C_in_drymass
      
      !canopy conductance
      canopy_cond = canopy_cond + a0*( GS_b1(p)*lai_grass(gn) + const0(p)*(2.0/eK(p))*const3 )
   endif
END DO

!Adjust canopy conductance
canopy_cond = canopy_cond / Max_loc / Max_loc

END SUBROUTINE photosynthesis



!**************************************************************************************************
! Optimum Leaf Area Index: daily computation using running mean of PAR
!**************************************************************************************************
SUBROUTINE lai_optimum &
    (lai_opt_grass, lai_opt_grass_RunningRecord, c3_no, c4_no, &
     pft_exist, psat, eK, lue, dlen, tmp_air, tmp_soil, par, par_grass)

USE data_structure
implicit none
common PN_s, PN_r

!Define common variables
real,dimension(PFT_no)::PN_s !proportion of nitrogen:stem (gN / gBiomass)
real,dimension(PFT_no)::PN_r !proportion of nitrogen:root (gN / gBiomass)

!Define argments
real   ,intent(IN)::dlen      !day length (hour)
real   ,intent(IN)::tmp_air   !2m air temperature (Celcius)
real   ,intent(IN)::tmp_soil  !soil temperature at 10 cm  depth (Celcius)
real   ,intent(IN)::par       !PAR of mid-day (micro mol photon m-2 s-1)
real   ,intent(IN)::par_grass !PAR on top of the grass layer (micro mol photon m-2 s-1)
integer,intent(IN)::c3_no     !PFT number of C3 grass
integer,intent(IN)::c4_no     !PFT number of C4 grass

logical,dimension(PFT_no),intent(IN)::pft_exist !PFT exist or not (.TRUE. or .FALSE.)
real   ,dimension(PFT_no),intent(IN)::psat      !light-saturated rate (micro mol CO2 m-2 s-1)
real   ,dimension(PFT_no),intent(IN)::eK        !light attenuation coefficient, no dimension
real   ,dimension(PFT_no),intent(IN)::lue       !monthly quantum yield (mol CO2 mol photon-1)

real,dimension(1:2)     ,intent(OUT)  ::lai_opt_grass  !Optimum LAI of grass layer (m2/m2)
real,dimension(1:2,1:20),intent(INOUT)::lai_opt_grass_RunningRecord !RunningRecord of lai_opt_grass

!Define local variables
real,dimension(PFT_no)::lai_opt !Optimum LAI for each PFT (m2/m2)
real    qt                      !QT10 of maintenance respiration
real    tmp_sensibility_air     !temperature sensibility multiplier for foliage and stem
real    tmp_sensibility_soil    !temperature sensibility multiplier for root
real    a0, a1, a2, a3          !for general usage
integer pft                     !loop counter
real    const1, const2          !temporal usage

!Prepare temperature sensibility
   qt                   = 2.0 * exp( -0.009 * (tmp_air - 15.0) )
   tmp_sensibility_air  = exp( (tmp_air  - 15.0) * log(qt) / 10.0 )
   qt                   = 2.0 * exp( -0.009 * (tmp_soil - 15.0) )
   tmp_sensibility_soil = exp( (tmp_soil - 15.0) * log(qt) / 10.0 )

!Each PFT procedures
lai_opt(:) = 0.0
do pft = 1 , PFT_no
   if (dlen      <= 0.1   ) cycle
   if (psat(pft) <= 0.001 ) cycle
   if (lue(pft)  <= 0.001 ) cycle
   if (eK(pft)   <= 0.001 ) cycle
   if (par_grass <= 1.00  ) cycle
   
   !a0: maintenance respiration rate of leaf (gDM / gDM / day)
   a0 = RM(pft) * PN_f(pft) * tmp_sensibility_air
   
   !a1: turnover cost of leaf (g dm / m2 / day)
   !a1 = 0.0
   a1 = (TO_f(pft)/365.0) * RG_f(pft)
   
   !a2: maintenance respiration rate of root per unit leaf-mass (gDM / gDM / day)
   a2 = RM(pft) * PN_r(pft) * tmp_sensibility_soil / FR_ratio(pft)
   
   !a3: turnover cost of root (g dm / m2 / day)
   !a3 = 0.0
    a3 = (TO_r(pft)/365.0) * RG_r(pft) / FR_ratio(pft)
   
   !const1: daily maintenance cost of leaf (gDM / m2)
   const1 = (a0+a1+a2+a3)/SLA(pft)
   const1 = max(0.0, const1)
   
   !optimum leaf index
   const2 = 1.0 - const1 / ( 0.09093*dlen*psat(pft) )
   const2 = 1.0 / (const2*const2) - 1.0
   const2 = (psat(pft)/lue(pft)) * const2
   
   if ( const2>0.0 ) then
      lai_opt(pft) = ( log(par_grass/const2) ) / eK(pft)
      lai_opt(pft) = max(0.0, lai_opt(pft))
   else
      lai_opt(pft) = 0.0
   endif
   
end do

!Procedures for grass PFTs
   lai_opt_grass(1) = lai_opt(c3_no)
   lai_opt_grass(2) = lai_opt(c4_no)
   
   lai_opt_grass_RunningRecord(1, 2:20) = lai_opt_grass_RunningRecord(1, 1:19)
   lai_opt_grass_RunningRecord(1, 1)    = lai_opt_grass(1)
   
   lai_opt_grass_RunningRecord(2, 2:20) = lai_opt_grass_RunningRecord(2, 1:19)
   lai_opt_grass_RunningRecord(2, 1)    = lai_opt_grass(2)
   
END SUBROUTINE lai_optimum



!**************************************************************************************************
! Leaf phenology (daily computation)
!**************************************************************************************************
SUBROUTINE leaf_season &
            (LAT, doy, counter, c_uptake, c_emission, pool_litter, &
            tmp_air_RunningRecord, tmp_coldest_20yr_ave, c3_no, c4_no, &
            pft_exist, npp, npp_10d_ave, stat_water_RunningRecord, &
            phenology, pheno_clo, pheno_cls, pheno_gdd, pheno_ncd, &
            pft, tree_exist, la, mass_leaf, mass_stock, mass_available, &
            type_grass, lai_grass, gmass_leaf, gmass_stock, gmass_available, &
            lai_opt_grass, lai_opt_grass_RunningRecord)

USE data_structure
implicit none

!Define arguments
   real   ,intent(IN)   ::LAT         !latitude  (degree)
   integer,intent(IN)   ::doy         !loop for day of year (1-365)
   integer,intent(IN)   ::counter     !simulation day (1-Simulation_year*365)
   real   ,intent(INOUT)::c_uptake    !monthly carbon uptake   (C g / month stand)
   real   ,intent(INOUT)::c_emission  !monthly carbon emission (gC/month/stand)
   
   real   ,intent(INOUT)::pool_litter !litter pool in a stand (g drymatter / stand)
   
   real,dimension(365),intent(IN)::tmp_air_RunningRecord !365days running record  of tmp_air
   real               ,intent(IN)::tmp_coldest_20yr_ave  !20yr running average of tmp_coldest_month
   
   logical,dimension(PFT_no)    ,intent(IN)   ::pft_exist   !PFT exist or not (True or Faulse)
   real   ,dimension(PFT_no)    ,intent(INOUT)::npp         !NPP for each pft (g dm / stand / day)
   real   ,dimension(PFT_no)    ,intent(IN)   ::npp_10d_ave !10 days running average of NPP
   real   ,dimension(PFT_no,365),intent(IN)   ::stat_water_RunningRecord
   
   logical,dimension(PFT_no),intent(INOUT)::phenology !phenological status (True or Faulse)
   integer,dimension(PFT_no),intent(INOUT)::pheno_clo !counter-number of last leaf onset
   integer,dimension(PFT_no),intent(INOUT)::pheno_cls !counter-number of last leaf shedding
   real                     ,intent(IN)   ::pheno_gdd !GDD on 5 degree base
   integer                  ,intent(IN)   ::pheno_ncd !Number of chilling day
   
   integer,intent(IN)::c3_no           !PFT number of C3 grass
   integer,intent(IN)::c4_no           !PFT number of C4 grass
   
   integer,dimension(1:Max_no),intent(IN)   ::pft            !Plant FUnctional Type
   logical,dimension(1:Max_no),intent(IN)   ::tree_exist     !
   real   ,dimension(1:Max_no),intent(INOUT)::la             !leaf area   (m2)
   real   ,dimension(1:Max_no),intent(INOUT)::mass_leaf      !foliage mass (g)
   real   ,dimension(1:Max_no),intent(INOUT)::mass_stock     !stock mass (g)
   real   ,dimension(1:Max_no),intent(INOUT)::mass_available !mass available (g)
   
   integer                 ,intent(IN)   ::type_grass      !grass type priority (3:C3, 4:C4)
   real,dimension(1:2)     ,intent(INOUT)::lai_grass       !leaf area index of grass (m2/m2)
   real,dimension(1:2)     ,intent(INOUT)::gmass_leaf      !foliage mass of grass    (g/m2)
   real,dimension(1:2)     ,intent(INOUT)::gmass_stock     !stock mass of grass      (g/m2)
   real,dimension(1:2)     ,intent(INOUT)::gmass_available !available mass of grass  (g/m2)
   real,dimension(1:2)     ,intent(IN)   ::lai_opt_grass   !Optimum LAI for grass   (m2/m2)
   real,dimension(1:2,1:20),intent(IN)   ::lai_opt_grass_RunningRecord
   
!Define local variables
   real    stockmass_min_tree  !minimum stock mass for dormance (g dm/individual)
   real    stockmass_min_grass !minimum stock mass for dormance (g dm/m2)
   
   real tmp_air_10d_ave !1- days running mean of air temperature (Celcius)
   
   integer i, no  !loop counter
   logical flag   !flag for phenology change
   integer j,k,l  !temporal integer value
   real    x      !temporal real value
   
   real mass_assm    !biomass assimirated (g DM/day/stand)
   real mass_combust !biomass combust     (g DM/day/stand)
   real mass_total   !total biomass before procedure --for checker-- (g DM/stand)
   real a1, a2, a3   !for general usage
   
!set parameters
   stockmass_min_tree  = 10.0
   stockmass_min_grass = 50.0
!prepare climatic factor
   tmp_air_10d_ave = sum( tmp_air_RunningRecord(1:10) ) / 10.0
   
!for carbon leaking checker
   a1 = gmass_leaf(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      a3 = (0.9*a1 + 0.1*a2) * (Max_loc**2)
   else
      a3 = (0.1*a1 + 0.9*a2) * (Max_loc**2)
   end if
   mass_total = a3 + sum(mass_leaf)+sum(mass_available)+sum(mass_stock) + pool_litter
   
!For every available PFTs
mass_assm    = 0.000
mass_combust = 0.000

DO i = 1, PFT_no
IF (.not. pft_exist(i))     cycle
IF (Phenology_type(i) == 0) cycle
    
    flag = .false.
   
   !************ On foliation phase *****************************************
   IF (phenology(i)) then
    
    !----------------- Checker (Foliage -> Domance) ------------------------
    select case ( Phenology_type(i))
      
      case (1) !boreal summer green needle leaf
        x = tmp_air_10d_ave
        if ( (x<2.0) .or. (x<10.0+tmp_coldest_20yr_ave) )          flag=.true.
        
      case (2:3) !other temperature-controlling woody PFTs
        x = tmp_air_10d_ave
        if ( (x<9.0) .or. (x<10.0+tmp_coldest_20yr_ave) )          flag=.true.
        
      case (4) !water-controlling PFTs (Botta et al. 2000)
        if ( npp_10d_ave(i) <= 0.0 )                               flag=.true.
        
      case (5) !C3 grass
        if ( sum(lai_opt_grass_RunningRecord(1,1:14))/14.0 < 0.1 ) flag=.true.
        !x = tmp_air_10d_ave
        !if ( x<5.0+tmp_coldest_20yr_ave .and. x<10.0 )             flag=.true.
        
      case (6) !C4 grass
        if ( sum(lai_opt_grass_RunningRecord(2,1:14))/14.0 < 0.1 ) flag=.true.
       !x = tmp_air_10d_ave
       !if ( x<5.0+tmp_coldest_20yr_ave .and. x<10.0 )             flag=.true.
        
    end select
    if ( counter - pheno_clo(i) < 60 ) flag=.false.
    !-----------------------------------------------------------------------
    
    !----------------- Procedure (Foliage -> Dormance) ---------------------
    if (flag) then
      
      if (Logging==1) write (*,*) 'Foliage phase ---> Domance phase', i, doy
      phenology(i) = .false.
      pheno_cls(i) = counter
      
      !for woody PFTs
      if (Life_type(i)==0) then
         do no=1, Max_no
         if (tree_exist(no) .and. pft(no)==i) then
             
              pool_litter = pool_litter + mass_leaf(no)
              mass_leaf(no)  = 0.0
              la(no)      = 0.0
              
              !give minimum stock mass
              if (mass_stock(no) < stockmass_min_tree) then
                pool_litter    = pool_litter + mass_stock(no)
                pool_litter    = pool_litter - stockmass_min_tree
                mass_stock(no) = stockmass_min_tree
              end if
              
         end if
         end do
      
      !for C3 grass PFTs
     elseif (i==c3_no) then
          if (type_grass==3) then; x = 0.9 * (Max_loc**2)
          else                   ; x = 0.1 * (Max_loc**2)
          endif
          
          pool_litter     = pool_litter + gmass_leaf(1) * x
          gmass_leaf(1) = 0.0
          lai_grass   (1) = 0.0
          
          !give minimum stock mass
          if (gmass_stock(1) < stockmass_min_grass) then
             pool_litter         = pool_litter - (stockmass_min_grass-gmass_stock(1)) * x
             gmass_stock(1) = stockmass_min_grass
             
             if ( pool_litter < 0.0 ) then
                mass_assm   = mass_assm - pool_litter
                pool_litter = 0.0
             endif
             
          end if
          
      !for C4 grass PFTs
     else
          if (type_grass==4) then; x = 0.9 * (Max_loc**2)
          else                   ; x = 0.1 * (Max_loc**2)
          endif
          
          pool_litter     = pool_litter + gmass_leaf(2) * x
          gmass_leaf(2) = 0.0
          lai_grass   (2) = 0.0
          
          !give minimum stock mass
          if (gmass_stock(2) < stockmass_min_grass) then
             pool_litter         = pool_litter - (stockmass_min_grass-gmass_stock(2)) * x
             gmass_stock(2) = stockmass_min_grass
             
             if ( pool_litter < 0.0 ) then
                mass_assm   = mass_assm - pool_litter
                pool_litter = 0.0
             endif
             
          end if
          
      endif
      
   endif
    !-----------------------------------------------------------------------
    
   !************ On dormance phase ****************************************
   ELSE
    
    !----------------- Checker (Dormance -> Foliage) -----------------------
    !                 [ Botta et al 2000 procedure ]                        
    select case ( Phenology_type(i) )
      
      case (1) !boreal deciduous needle-leaved-woods
        j = 0
        do k = 1, 60
            if (tmp_air_RunningRecord(k) > -5.0) j=j+1
        end do
        
        if ( j >= 15 )                                                      flag = .true. 
        if ( LAT>=0.0 .and. (doy< 30+LAT .or. doy>130+LAT) ) flag = .false.
        if ( LAT< 0.0 .and. (doy<212-LAT .or. doy>312-LAT) ) flag = .false.
        
      case (2:3) !other temperature-controlling PFTs
        if ( pheno_gdd > -68.0 + 638.0 * exp(-0.01*pheno_ncd) )             flag = .true. 
        if ( LAT>=0.0 .and. (doy< 30+LAT .or. doy>130+LAT) ) flag = .false.
        if ( LAT< 0.0 .and. (doy<212-LAT .or. doy>312-LAT) ) flag = .false.
        
      case (4) !water-controlling deciduous
        if ( sum( stat_water_RunningRecord(i,51:55) ) > &
             sum( stat_water_RunningRecord(i,56:60) )   )   flag = .true.
        
        j = 0
        do k = 25, 50
           if ( stat_water_RunningRecord(i,k) > stat_water_RunningRecord(i,k+1) ) j=j+1
        end do
        if ( j<13 )                                                         flag = .false.
        
      case (5) !C3grass
        if ( sum(lai_opt_grass_RunningRecord(1,1:14)) / 14.0 > 0.1 )        flag = .true. 
       !if ( LAT>=0.0 .and. (doy< 30+LAT .or. doy>130+LAT) ) flag = .false.
       !if ( LAT< 0.0 .and. (doy<212-LAT .or. doy>312-LAT) ) flag = .false.
        
      case (6) !C4grass
        if ( sum(lai_opt_grass_RunningRecord(2,1:14)) / 14.0 > 0.1 )        flag = .true. 
       !if ( LAT>=0.0 .and. (doy< 30+LAT .or. doy>130+LAT) ) flag = .false.
       !if ( LAT< 0.0 .and. (doy<212-LAT .or. doy>312-LAT) ) flag = .false.
        
    end select
    if ( counter - pheno_cls(i) < 60 ) flag=.false.
    
    !-----------------------------------------------------------------------
    
    !----------------- Procedure (Dormance -> Foliage) ---------------------
    if (flag) then
        if (Logging==1) write (*,*) 'Domance phase ---> Foliage phase', i, doy
        phenology(i) = .true.
        pheno_clo(i) = counter
    endif
    !-----------------------------------------------------------------------
    
    ENDIF
    
   !************ gradual release of stock energy (trees)  *****************
    l = 14 !day lengh required for full foliation for woody PFTs
    IF (Life_type(i) == 0)        then
    IF (pheno_clo(i)+l > counter) then
        do no=1, Max_no
        if (tree_exist(no) .and. pft(no)==i) then
            !x: stock mass for releasing in the day (g)
            x = mass_stock(no)/(l+pheno_clo(i)-counter)
            
            !release process
            mass_available (no) = mass_available(no) + x / RG_stock_out
            mass_stock     (no) = mass_stock    (no) - x
            mass_combust        = mass_combust       + (x - x / RG_stock_out)
            npp            (i)  = npp(i)             - (x - x / RG_stock_out)
        endif
        enddo
    ENDIF
    ENDIF
    
   !************ gradual release of stock energy (C3 grass)  *****************
    l = 14 !day lengh required for full foliation for C3 grass
    IF ( i == c3_no                  ) then
    IF ( pheno_clo(i)+l   >  counter ) then
    IF ( phenology(c3_no)            ) then
       
       !x: stock mass consumed within a day (g/m2)
       x = gmass_stock(1) / (l+pheno_clo(c3_no)-counter)
       
       !a1: fraction of cover on forest floor
       if (type_grass==3) then 
         a1 = 0.9
       else
         a1 = 0.1
       endif
       
       !release process
       gmass_available(1) = gmass_available(1) + x / RG_stock_out
       gmass_stock(1)     = gmass_stock(1)     - x
       mass_combust       = mass_combust  + (x - x / RG_stock_out)*Max_loc*Max_loc*a1
       npp(c4_no)         = npp(c4_no)    - (x - x / RG_stock_out)*Max_loc*Max_loc*a1
       
    ENDIF
    ENDIF
    ENDIF
    
   !************ gradual release of stock energy (C4 grass)  *****************
    l = 14 !day lengh required for full foliation for C4 grass
    IF ( i == c4_no                   ) then
    IF ( pheno_clo(c4_no)+l > counter ) then
    IF ( phenology(c4_no)             ) then
       
       !x: stock mass consumed within a day (g/m2)
       x = gmass_stock(2) / (l+pheno_clo(c4_no)-counter)
       
       !a1: coverage proportion
       if (type_grass==4) then 
         a1 = 0.9
       else
         a1 = 0.1
       endif
       
       !release process
       gmass_available(2) = gmass_available(2) + x / RG_stock_out
       gmass_stock(2)     = gmass_stock(2)     - x
       mass_combust       = mass_combust  + (x - x / RG_stock_out)*Max_loc*Max_loc*a1
       npp(c4_no)         = npp(c4_no)    - (x - x / RG_stock_out)*Max_loc*Max_loc*a1
       
    ENDIF
    ENDIF
    ENDIF
    
END DO

!carbon leak checker
   a1 = gmass_leaf(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      a3 = (0.9*a1 + 0.1*a2) * (Max_loc**2)
   else
      a3 = (0.1*a1 + 0.9*a2) * (Max_loc**2)
   end if
   a3 = a3 + sum(mass_leaf)+sum(mass_available)+sum(mass_stock) + pool_litter
   a3 = a3 + mass_combust - mass_assm
   
   if ( abs(mass_total-a3)>20.0 .and. Logging==1 ) then
      write (*,*) 'Error @ Leaf_season: leaking mass (g) ->', doy, mass_total-a3
   endif
   
!Total carbon emission by maintenance respiration
   c_emission = c_emission + mass_combust * C_in_drymass
   c_uptake   = c_uptake   + mass_assm    * C_in_drymass
   
END SUBROUTINE leaf_season



!**************************************************************************************************
! Maintenance respiration (daily computation)
!**************************************************************************************************
SUBROUTINE maintenance_resp &
        (doy, pft, height, bole, tree_exist, dbh_heartwood, dbh_sapwood, la, &
        mass_leaf, mass_trunk, mass_root, mass_available, mass_stock, &
        stat_leaf_crownbottom, stat_leaf_crowntop, stat_leaf, &
        type_grass, lai_grass, c3_no, c4_no, &
        gmass_leaf, gmass_root, gmass_available, gmass_stock, &
        pool_litter, c_emission, phenology, npp, mort_regu1, tmp_air, tmp_soil )

USE data_structure
implicit none
common PN_s, PN_r

!PFT specific constants
real,dimension(PFT_no)::PN_s !proportion of nitrogen:stem (gN / gBiomass)
real,dimension(PFT_no)::PN_r !proportion of nitrogen:root (gN / gBiomass)

!Define arguments
integer,intent(IN)::doy     !day of the year (1-365)

real,intent(IN)   ::tmp_air     !air  temperature (Cecius)
real,intent(IN)   ::tmp_soil    !soil temperature (Cecius)
real,intent(INOUT)::pool_litter !litter pool in a stand (g / stand)
real,intent(INOUT)::c_emission  !monthly carbon emission (C g / month stand)

logical,dimension(PFT_no)  ,intent(IN)   ::phenology  !phenological status
real   ,dimension(PFT_no)  ,intent(INOUT)::npp        !NPP for each pft (g dm / stand / day)
real   ,dimension(1:Max_no),intent(INOUT)::mort_regu1 !annual NPP (g dm)

integer,dimension(1:Max_no)   ,intent(IN)   ::pft        !Plant Functional Type (1-PFT_no)
integer,dimension(1:Max_no)   ,intent(IN)   ::height     !tree height from 1.3m above ground (STEP)
integer,dimension(1:Max_no)   ,intent(IN)   ::bole       !bole length from 1.3m above ground (STEP)
logical,dimension(1:Max_no)   ,intent(IN)   ::tree_exist     !tree exist or not
real   ,dimension(1:Max_no)   ,intent(INOUT)::dbh_heartwood  !heartwood diameter at 1.3m (m)
real   ,dimension(1:Max_no)   ,intent(INOUT)::dbh_sapwood    !sapwood diameter at 1.3m   (m)
real   ,dimension(1:Max_no)   ,intent(INOUT)::la             !leaf area (m2)
real   ,dimension(1:Max_no)   ,intent(INOUT)::mass_leaf      !biomass of foliage (g)
real   ,dimension(1:Max_no)   ,intent(IN)   ::mass_trunk     !biomass of stem (g)
real   ,dimension(1:Max_no)   ,intent(INOUT)::mass_root      !biomass of root (g)
real   ,dimension(1:Max_no)   ,intent(INOUT)::mass_available !biomass available (g)
real   ,dimension(1:Max_no)   ,intent(INOUT)::mass_stock     !biomass for stock (g)
real   ,dimension(1:Max_no,10),intent(INOUT)::stat_leaf_crownbottom !annual NPP of foliage bottom
real   ,dimension(1:Max_no   ),intent(INOUT)::stat_leaf_crowntop    !annual NPP of foliage top
real   ,dimension(1:Max_no   ),intent(INOUT)::stat_leaf             !benefit of producing new leaf mass (g/g Leaf)

integer,intent(IN)::c3_no           !PFT number of C3 grass
integer,intent(IN)::c4_no           !PFT number of C4 grass

integer                ,intent(IN)  ::type_grass      !grass priority (3:C3 type, 4:C4 type)
real   ,dimension(1:2),intent(INOUT)::lai_grass       !leaf area index of grass (m2/m2)
real   ,dimension(1:2),intent(INOUT)::gmass_leaf      !foliage mass of grass (g/m2)
real   ,dimension(1:2),intent(INOUT)::gmass_root      !root mass of root (g/m2)
real   ,dimension(1:2),intent(INOUT)::gmass_available !available mass of grass (g/m2)
real   ,dimension(1:2),intent(INOUT)::gmass_stock     !herbaceous stock biomass (g/m2)

!Define local variables
real    qt                    !QT
real    tmp_sensibility_air   !temperature sensibility multiplier for foliage and stem
real    tmp_sensibility_soil  !temperature sensibility multiplier for root
real    resp_15C_air          !maintenance respiration mass of organ in air  at 15C (g)
real    resp_15C_soil         !maintenance respiration mass at organ in soil at 15C (g)
real    mass_required         !resource required for maintenance respiration (g)
real    mass_resp             !resource usage for mainatenance respiration (g)

real    proportion_of_sapwood !sapwood_mass / stem_mass
real    frac_organ_remove     !daily proportion biomass remove
                              !when resource for maintenance respiration resource absent

real    mass_total       !total biomass before procedure --for checker-- (g DM/stand)
real    mass_combust     !biomass combust (g DM/day/stand)
real    mass_resp_grass  !respiration by c3 grass (g dm / m2 / day)
real    litter_sum_grass !mass of litter production by c3 grass (g dm/m2/day)

integer no, i, p, gn  !loop counter
real::a1, a2, a3      !for general usage
real::x, y, z         !for general usage

!**Record total carbon in this virtual forest for carbon-leak-chacking
   !woody PFTs
   mass_total = sum(mass_leaf)+sum(mass_root)+sum(mass_available)+sum(mass_stock)+pool_litter
   !grass PFTs
   a1 = gmass_leaf(1) + gmass_root(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_root(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      a3 = (0.90*a1 + 0.10*a2) * (Max_loc**2)
   else
      a3 = (0.10*a1 + 0.90*a2) * (Max_loc**2)
   end if
   mass_total = mass_total + a3
   
!*** Set initial values and 
   !dry-mass to combust (g dm / stand)
   mass_combust = 0.000
   
   !set parameter
   frac_organ_remove = 0.03
   
   !temperature sensibility, for foliage and sapwood
       !for leaves and stem
   qt                   = 2.0 * exp( -0.009 * (tmp_air-15.0) )
   tmp_sensibility_air  = exp( (tmp_air-15.0) * log(qt) / 10.0 )
       !for root
   qt                   = 2.0 * exp( -0.009 * (tmp_soil-15.0) )
   tmp_sensibility_soil = exp( (tmp_soil - 15.0) * log(qt) / 10.0 )

!*** Woody PFTs
DO no=1, Max_no
If ( tree_exist(no)      ) then  !when tree exist
   
   !set PFT number
   p = pft(no)
   
   !Calculate daily resource required for maintenance respiration
   proportion_of_sapwood = 1.0-(dbh_heartwood(no)**2)/( (dbh_sapwood(no)+dbh_heartwood(no))**2 ) 
   resp_15C_air         = RM(p) * PN_f(p) * mass_leaf(no) + &
                          RM(p) * PN_s(p) * mass_trunk(no) * proportion_of_sapwood
   resp_15C_soil        = RM(p) * PN_r(p) * mass_root(no)
   mass_required        = resp_15C_air  * tmp_sensibility_air  + &
                          resp_15C_soil * tmp_sensibility_soil    
   
   !phase 1: use mass_available
   mass_resp            = min(mass_required, mass_available(no))
   mass_required        = mass_required       - mass_resp
   mass_available(no)   = mass_available(no)  - mass_resp
   mass_combust         = mass_combust        + mass_resp
   npp(p)               = npp(p)             - mass_resp
   mort_regu1(no)       = mort_regu1(no)     - mass_resp
   
   !phase 2: use mass_stock
   if (mass_required > 0.001) then
      mass_resp            = min(mass_required, mass_stock(no)/ RG_stock_out)
      mass_required        = mass_required  - mass_resp
      mass_stock(no)       = mass_stock(no) - mass_resp * RG_stock_out
      mass_combust         = mass_combust   + mass_resp * RG_stock_out
      npp(p)               = npp(p)         - mass_resp * RG_stock_out
      mort_regu1(no)       = mort_regu1(no) - mass_resp * RG_stock_out
   end if
   
   !phase 3: when resource absent, a portion of leaves drop
   if (mass_required > 0.001) then
      a1 = frac_organ_remove * mass_leaf(no)
      a2 = frac_organ_remove * mass_root(no)
      
      pool_litter   = pool_litter + a1 + a2
      mass_leaf(no) = mass_leaf(no)  - a1
      mass_root(no) = mass_root(no)  - a2
      la(no)        = mass_leaf(no) * SLA(p)
      
      x = dbh_sapwood(no)
      y = dbh_heartwood(no)
      z = sqrt( y*y + frac_organ_remove * ( x*x + 2.0*x*y ) ) - y
      dbh_sapwood(no) = dbh_sapwood(no) - z
      dbh_heartwood(no) = dbh_heartwood(no) + z
   end if
      
      !record crown-bottom-layer NPP
      a1 = mass_leaf(no) / ( height(no)-bole(no) )             !leaf mass (g/step)
      a2 = a1 * PN_f(p) * RM(p) * tmp_sensibility_air + &
           a1 * (TO_f(p)/365) * RG_f(p)               !leaf maintenance cost (g/day/step)
      a3 = a1 / FR_ratio(p) * PN_r(p) * RM(p) * tmp_sensibility_soil + &
           a1 / FR_ratio(p) * (TO_r(p)/365) * RG_r(p) !root maintenance cost (g/day/step)
      do i = 1, 10
      stat_leaf_crownbottom(no,i) = stat_leaf_crownbottom(no,i) - a2 - a3
      end do
      stat_leaf_crowntop(no)      = stat_leaf_crowntop(no)      - a2 - a3
      
      !benefit of producing new leaf mass (g / g Leaf)
      stat_leaf(no) = stat_leaf(no) - (a2+a3) / max(0.01,a1)
      
End if
End DO

!**Carbon-leak-checker1
   a3 = sum(mass_leaf)+sum(mass_root)+sum(mass_available)+sum(mass_stock) &
        + pool_litter + mass_combust
   
   a1 = gmass_leaf(1) + gmass_root(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_root(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      a3 = a3 + (0.90*a1 + 0.10*a2) * (Max_loc**2)
   else
      a3 = a3 + (0.10*a1 + 0.90*a2) * (Max_loc**2)
   end if
   
   if ( abs(mass_total-a3)>20.0 .and. Logging==1 ) then
       write (*,*) 'Error maintenance_respiration1: leaking mass (g) ->', abs(mass_total-a3), doy
   end if
   
!Herbaceous PFTs
DO gn=1, 2 !( gn1->C3, gn2->C4 )
   !reset mass respiration 
   mass_resp_grass  = 0.000
   
   !set pft number, p
   if (gn==1) then
      p = c3_no
   else
      p = c4_no
   endif
   
   !calculate daily mass_required for maintenance respiration (g/m2)
   resp_15C_air  = RM(p) * PN_f(p) * gmass_leaf(gn)
   resp_15C_soil = RM(p) * PN_r(p) * gmass_root(gn)
   mass_required = resp_15C_air*tmp_sensibility_air + resp_15C_soil*tmp_sensibility_soil
   
   !Source 1: mass_available
   x                  = min(mass_required, gmass_available(gn))
   mass_required      = mass_required       - x
   gmass_available(gn)= gmass_available(gn) - x
   mass_resp_grass    = x
   
   !Source 2: mass_stock
   if (mass_required > 0.001) then
    x               = min(mass_required, gmass_stock(gn) / RG_stock_out)
    mass_required   = mass_required   - x
    gmass_stock(gn) = gmass_stock(gn) - x * RG_stock_out
    mass_resp_grass = mass_resp_grass + x * RG_stock_out
   endif
    
   !Source 3: when resource absent
   litter_sum_grass     = 0.0
   if (mass_required > 0.001) then
    litter_sum_grass   = frac_organ_remove * ( gmass_leaf(gn) + gmass_root(gn) )
    gmass_leaf(gn)     = (1.0 - frac_organ_remove) * gmass_leaf(gn)
    gmass_root(gn)     = (1.0 - frac_organ_remove) * gmass_root(gn)
    lai_grass(gn)      = gmass_leaf(gn) * SLA(p)
   endif
   
   !carbon balance
                                  a1 = 0.100 * (Max_loc**2)
   if (gn==1 .and. type_grass==3) a1 = 0.900 * (Max_loc**2)
   if (gn==2 .and. type_grass==4) a1 = 0.900 * (Max_loc**2)
   
   mass_combust = mass_combust + mass_resp_grass  * a1
   npp(p)       = npp(p)       - mass_resp_grass  * a1
   pool_litter  = pool_litter  + litter_sum_grass * a1
   
END DO

!**Carbon-leak-checker2
   a3 = sum(mass_leaf)+sum(mass_root)+sum(mass_available)+sum(mass_stock) &
        + pool_litter + mass_combust
   
   a1 = gmass_leaf(1) + gmass_root(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_root(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      a3 = a3 + (0.9*a1 + 0.1*a2) * (Max_loc**2)
   else
      a3 = a3 + (0.1*a1 + 0.9*a2) * (Max_loc**2)
   end if
   
   if ( abs(mass_total-a3)>20.0 .and. Logging==1 ) then
       write (*,*) 'Error maintenance_respiration2: leaking mass (g) ->', abs(mass_total-a3), doy
   end if
   
!Total carbon emission by maintenance respiration
   c_emission = c_emission + mass_combust * C_in_drymass
   
END SUBROUTINE maintenance_resp



!**************************************************************************************************
! Turnover (daily computation)
!**************************************************************************************************
SUBROUTINE turnover &
         (tree_exist, pft, mass_leaf, mass_root, dbh_heartwood, dbh_sapwood, la, &
          lai_grass, gmass_leaf, gmass_root, type_grass, &
          pool_litter, counter, pheno_clo, pheno_cls, c3_no, c4_no )

USE data_structure
implicit none

!Define arguments
real                     ,intent(INOUT)::pool_litter !litter pool in a stand (g / stand )
integer                  ,intent(IN)   ::counter     !simulation day (1-Simulation_year*365)
integer,dimension(PFT_no),intent(IN)   ::pheno_clo   !counter number of last leaf onset
integer,dimension(PFT_no),intent(IN)   ::pheno_cls   !counter number of last leaf shedding
integer                  ,intent(IN)   ::c3_no       !PFT number of C3 grass
integer                  ,intent(IN)   ::c4_no       !PFT number of C4 grass

logical,dimension(1:Max_no),intent(IN)   ::tree_exist    !tree exist or not (.TRUE. or .FALSE.)
integer,dimension(1:Max_no),intent(IN)   ::pft           !Plant Functional Type (1-PFT_no)
real   ,dimension(1:Max_no),intent(INOUT)::mass_leaf     !biomass of foliage (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_root     !biomass of root (g)
real   ,dimension(1:Max_no),intent(INOUT)::dbh_heartwood !heartwood diameter at 1.3m (m)
real   ,dimension(1:Max_no),intent(INOUT)::dbh_sapwood   !sapwood diameter at 1.3m   (m)
real   ,dimension(1:Max_no),intent(INOUT)::la            !leaf area (m2)

integer               ,intent(IN)   ::type_grass !priority grass type (3:C3, 4:C4)
real   ,dimension(1:2),intent(INOUT)::lai_grass  !leaf area index of grass (m2/m2)
real   ,dimension(1:2),intent(INOUT)::gmass_leaf !grass foliage biomass in a unit area (g/m2)
real   ,dimension(1:2),intent(INOUT)::gmass_root !grass root  biomass in a unit area   (g/m2)

!Define local variables
real    t_rate      !daily turnover rate
real    t_mass      !daily turnover mass
real    mass_total  !for carbon leak chaker (g DM/stand)
integer p           !for PFT number
integer no, i, gn   !for loop counter
real    x, y, z     !for general usage
real    a1, a2      !for general usage

!carbon leak checker1
mass_total = sum(mass_leaf) + sum(mass_root) + pool_litter

!Woody PFTs
DO no=1, Max_no 
if (tree_exist(no)) then
    !set pft number
    p = pft(no)
    
   !foliage
    if ( Phenology_type(p)==0 .or. counter-pheno_clo(p)>365 ) then
        t_mass        = mass_leaf(no) * ( TO_f(p) / 365.0000 )
        mass_leaf(no) = mass_leaf(no)- t_mass
        pool_litter   = pool_litter  + t_mass
        la(no)        = mass_leaf(no) * SLA(p)
    end if
    
   !root
    t_mass        = mass_root(no) * ( TO_r(pft(no)) / 365.0000 )
    mass_root(no) = mass_root(no) - t_mass
    pool_litter   = pool_litter   + t_mass
   
   !sapwood
    x                 = dbh_sapwood(no)
    y                 = dbh_heartwood(no)
    z                 = TO_s(pft(no)) / 365.0000
    t_rate            = sqrt( y*y + z*( x*x + 2*x*y ) ) - y
    dbh_sapwood(no)   = dbh_sapwood(no) - t_rate
    dbh_heartwood(no) = dbh_heartwood(no) + t_rate
end if
END DO

!carbon leak checker1
   x = sum(mass_leaf) + sum(mass_root) + pool_litter
   if ( abs(mass_total-x)>200.0 .and. Logging==1 ) then
      write (*,*) 'Error turnover1: leaking mass (g) ->', mass_total-x
   end if

!carbon leak checker2
   if (type_grass==3) then
      a1 = 0.9*(gmass_leaf(1)+gmass_root(1)) + 0.1*(gmass_leaf(2)+gmass_root(2))
   else
      a1 = 0.1*(gmass_leaf(1)+gmass_root(1)) + 0.9*(gmass_leaf(2)+gmass_root(2))
   endif
   mass_total = a1*Max_loc*Max_loc + pool_litter

!Herbaceous PFTs
DO gn=1, 2
   !set pft number, p
   p = c4_no; if (gn==1) p = c3_no
   
   !a1: proportion of coverage
                                  a1=0.1
   if (gn==1 .and. type_grass==3) a1=0.9
   if (gn==2 .and. type_grass==4) a1=0.9
   
   !foliage turnover
   ! (for evergreen PFTs) or (for water controlling PFT without leaf shedding)
   if ( counter-pheno_cls(p)>365 ) then
     t_rate          = TO_f(p) / 365.0000
     pool_litter     = pool_litter  + gmass_leaf(gn)*t_rate*Max_loc*Max_loc*a1
     gmass_leaf(gn)  = gmass_leaf(gn) - gmass_leaf(gn)*t_rate
     lai_grass(gn)   = gmass_leaf(gn) * SLA(p)
   end if
   
   !root turnover
      t_rate         = TO_r(p) / 365.0000
      pool_litter    = pool_litter    + gmass_root(gn)*t_rate*Max_loc*Max_loc*a1
      gmass_root(gn) = gmass_root(gn) - gmass_root(gn)*t_rate
END DO

!carbon leak checker2
   if (type_grass==3) then
      a1 = 0.9*(gmass_leaf(1) + gmass_root(1)) + 0.1*(gmass_leaf(2) + gmass_root(2))
   else
      a1 = 0.1*(gmass_leaf(1) + gmass_root(1)) + 0.9*(gmass_leaf(2) + gmass_root(2))
   endif
   a2 = mass_total - (a1*Max_loc*Max_loc + pool_litter)
   
   if ( abs(a2)>200.0 .and. Logging==1 ) then
      write (*,*) 'Error turnover2: leaking mass (g) ->', a2
   end if

END SUBROUTINE turnover



!**************************************************************************************************
! growth woody PFTs (daily computation)
!**************************************************************************************************
SUBROUTINE growth_wood &
   (doy, phenology, pheno_clo, stat_leaf_RunningRecord, counter, &
   pft, height, bole, dbh_heartwood, dbh_sapwood, crown_area, crown_diameter, tree_exist, &
   la, mort_regu2, mass_leaf, mass_root, mass_available, mass_stock, &
   pool_litter, c_emission, npp, mort_regu1 )

USE data_structure
implicit none

!Define arguments
integer                        ,intent(IN)::doy        !day of the year (1-365)

logical,dimension(PFT_no)      ,intent(IN)::phenology  !phenological status
integer,dimension(PFT_no)      ,intent(IN)::pheno_clo  !counter number of last leaf onset
real   ,dimension(1:Max_no,1:7),intent(IN)::stat_leaf_RunningRecord
integer                        ,intent(IN)::counter    !simulation day (1-Simulation_year*365)

integer,dimension(1:Max_no),intent(IN)::pft            !Plant Functional Type (1-PFT_no)
integer,dimension(1:Max_no),intent(IN)::height         !Tree height from 1.3m above ground   (STEP)
integer,dimension(1:Max_no),intent(IN)::bole           !Bole length (STEP)
real   ,dimension(1:Max_no),intent(IN)::dbh_heartwood  !Hertwood diameter at 1.3m (m)
real   ,dimension(1:Max_no),intent(IN)::dbh_sapwood    !Sapwood diameter at 1.3m (m)
real   ,dimension(1:Max_no),intent(IN)::crown_area     !Crown area (m2)
real   ,dimension(1:Max_no),intent(IN)::crown_diameter !crown diameter (m)

logical,dimension(1:Max_no),intent(IN)::tree_exist !tree exist or not (.TRUE. or .FALSE.) 

real,dimension(1:Max_no),intent(INOUT)::la             !Leaf area         (m2)
real,dimension(1:Max_no),intent(INOUT)::mort_regu2     !average leaf area of last year (m2/day)
real,dimension(1:Max_no),intent(INOUT)::mass_leaf      !biomass of foliage (g)
real,dimension(1:Max_no),intent(INOUT)::mass_root      !biomass of root    (g)
real,dimension(1:Max_no),intent(INOUT)::mass_available !biomass available  (g)
real,dimension(1:Max_no),intent(INOUT)::mass_stock     !biomass for stock  (g)

real                    ,intent(INOUT)::pool_litter    !litter pool in a stand (g DM / stand)
real                    ,intent(INOUT)::c_emission     !carbon emission (g C / month / stand)
real,dimension(PFT_no)  ,intent(INOUT)::npp            
real,dimension(1:Max_no),intent(INOUT)::mort_regu1     !annual NPP (g dm)

!Define local variables
integer p              !for PFT number
integer no             !for loop counter
real    a1, a2, a3, a4 !for temporal usage
real    mass_combust   !biomass combust (g DM/day/stand)
real    mass_total     !total biomass before procedure --for checker-- (g DM/stand)

!Initialize
mass_combust = 0.000
mass_total   = sum(mass_leaf)+sum(mass_root)+sum(mass_available)+sum(mass_stock)+pool_litter

!Each individual of woody PFT procedure
DO no=1, Max_no 
IF ( .not. tree_exist(no)     ) cycle
IF ( .not. phenology(pft(no)) ) cycle
    
    !SET PFT NUMBER
   p = pft(no)
    
    !ROOT MASS SUPPLEMENT
    !a1: required root mass (g)
    a1 = max( 0.00000, mass_leaf(no)/FR_ratio(p) - mass_root(no) )
    !a2: maximum root mass increament (g)
    a2 = mass_available(no) / RG_r(p)
    !a3 = actual increament of Root mass with considering resource_available
    a3 = min( a1, a2 )
    !root mass supplement precedure
    mass_root(no)         = mass_root(no)   +  a3               
    mass_available(no) = mass_available(no) -  a3 * RG_r(p)     
    mass_combust       = mass_combust       + (a3 * RG_r(p) - a3)
    npp(p)             = npp(p)             - (a3 * RG_r(p) - a3)
    mort_regu1(no)     = mort_regu1(no)     - (a3 * RG_r(p) - a3)
    
    !STOCK RESOURCE SUPPLEMENT
    if (counter > pheno_clo(p)+30) then !after more than 30 days from foliation
       !a1 = resource requirement for stock (g)
      a1 = max( 0.00000, mass_leaf(no) - mass_stock(no) )
       !a2 = maximum increment of mass_stock (g)
      a2 = mass_available(no) / RG_stock_in
       !a3:actual increment of mass_stock (g/m2)
       a3 = min( a1, a2 )
       !stock resource supplement precedure
       mass_stock(no)     = mass_stock(no)     +  a3                   
       mass_available(no) = mass_available(no) -  a3 * RG_stock_in     
       mass_combust       = mass_combust       + (a3 * RG_stock_in - a3)
       npp(p)             = npp(p)             - (a3 * RG_stock_in - a3)
       mort_regu1(no)     = mort_regu1(no)     - (a3 * RG_stock_in - a3)
    end if
    
    !FOLIAGE MASS SUPPLEMENT
!    if ( la(no)/crown_area(no) < 1.0 .or. sum(stat_leaf_RunningRecord(no,:)) > 0.0 ) then
       !a1: foliage mass that fullfill existing crown size (g)
!       a1 = crown_area(no) * (height(no)-bole(no)) * STEP * LAD_max(p) / SLA(p)
       a1 = crown_area(no) + PI * crown_diameter(no) * real(height(no)-bole(no)) * STEP
       a1 = a1 * LA_max(p) / SLA(p)
       !a2: foliage mass that fullfill existing Sapwood (g)
       a2 = ALM1(p) * (1/SLA(p)) &
           * (PI/4) * (dbh_sapwood(no)**2 + 2*dbh_sapwood(no)*dbh_heartwood(no))
       !a3: foliage mass that can increase with resource available (g)
       a3 = mass_available(no) / RG_f(p)
       !a4: required foliage mass (g)
       a4 = max(0.00000, min(a1, a2) - mass_leaf(no))
       !a4: actual increament (g)
       a4 = min( a3, a4 )
       if ( a4 > 0.0) then
           mass_leaf(no)         = mass_leaf(no)    +  a4               
           mass_available(no) = mass_available(no)  -  a4 * RG_f(p)     
           mass_combust       = mass_combust        +  a4 * RG_f(p) - a4
           npp(p)             = npp(p)              - (a4 * RG_f(p) - a4)
           mort_regu1(no)     = mort_regu1(no)      - (a4 * RG_f(p) - a4)
           la(no)             = mass_leaf(no) * SLA(p)
       end if
!    end if
    
    !update mort_regu2 (annumal maximum of leaf area)
    !mort_regu2(no) = max(mort_regu2(no), la(no))
    mort_regu2(no) = mort_regu2(no) + la(no)/365
    
END DO

!carbon leak checker
a1 = sum(mass_leaf)+sum(mass_root)+sum(mass_available)+sum(mass_stock)+pool_litter+mass_combust
a2 = abs(mass_total - a1)
if ( a2>100.0 .and. Logging==1 ) then
   write (*,*) 'Error @ growth_wood: leaking mass (g) ->', a2, 'doy ->',doy
end if

!carbon emission
c_emission = c_emission + mass_combust * C_in_drymass

End subroutine growth_wood



!**************************************************************************************************
! growth_grass (daily computation)
!**************************************************************************************************
SUBROUTINE growth_grass &
      (doy, counter, pool_litter, c_emission, npp, phenology, pheno_clo, &
      type_grass, lai_grass, gmass_leaf, gmass_root, gmass_available, &
      gmass_stock, lai_opt_grass_RunningRecord, c3_no, c4_no)

USE data_structure
implicit none

!Define arguments
integer,intent(IN)   ::doy         !day of the year (1~365)
integer,intent(IN)   ::counter     !simulation day (1~Simulation_year*365)
real   ,intent(INOUT)::pool_litter !litter pool in a stand (g / stand)
real   ,intent(INOUT)::c_emission  !monthly carbon emission (C g / month / stand)

integer,intent(IN)   :: c3_no      !PFT number of C3 grass
integer,intent(IN)   :: c4_no      !PFT number of C4 grass

real   ,dimension(PFT_no),intent(INOUT)::npp
logical,dimension(PFT_no),intent(IN)   ::phenology !phenological status
integer,dimension(PFT_no),intent(IN)   ::pheno_clo !counter number of last leaf onset
                                                   !running record of lai_opt_grass (m2/m2)

integer                  ,intent(IN)   ::type_grass      !grass type (3:C3, 4:C4)
real,dimension(1:2)      ,intent(INOUT)::lai_grass       !leaf area index of grass (m2/m2)
real,dimension(1:2)      ,intent(INOUT)::gmass_leaf      !grass foliage   biomass  (g/m2)
real,dimension(1:2)      ,intent(INOUT)::gmass_root      !grass root      biomass  (g/m2)
real,dimension(1:2)      ,intent(INOUT)::gmass_available !grass available biomass  (g/m2)
real,dimension(1:2)      ,intent(INOUT)::gmass_stock     !grass stock     biomass  (g/m2)
real,dimension(1:2,1:20) ,intent(IN)   ::lai_opt_grass_RunningRecord

!Define local variables
real    lai_opt      !optimum LAI of grass layer (m2/m2)
real    mass_combust !biomass combust (g dm/day/stand)
real    mass_total   !total biomass before procedure --for checker-- (g dm/m2)
real    area_cover   !coverage area (m2)
integer p            !PFT number
integer i, gn        !for loop counter
real    a1, a2, a3   !for temporal usage

!carbon leak checker (before procedure)
   a1 = gmass_leaf(1) + gmass_root(1) + gmass_available(1) + gmass_stock(1)
   a2 = gmass_leaf(2) + gmass_root(2) + gmass_available(2) + gmass_stock(2)
   if (type_grass==3) then
      mass_total = (0.9*a1 + 0.1*a2)*Max_loc*Max_loc + pool_litter
   else
      mass_total = (0.1*a1 + 0.9*a2)*Max_loc*Max_loc + pool_litter
   end if
   
!initialize local variables
   mass_combust = 0.000
   
!each grass type procedure
DO gn = 1, 2
    
    !set pft number: p
    p=c4_no; if (gn==1) p=c3_no
    
IF ( phenology(p) ) then
    
    !set coverage area within the stand: area_cover (m2)
                                    area_cover = 0.1 * Max_loc * Max_loc
    if ( gn==1 .and. type_grass==3) area_cover = 0.9 * Max_loc * Max_loc
    if ( gn==2 .and. type_grass==4) area_cover = 0.9 * Max_loc * Max_loc
    
    !set lai_opt, running mean of optimum leaf area index
    lai_opt = sum(lai_opt_grass_RunningRecord(gn,1:7)) / 7.0
    
    !excess foliage senescence procedure (optional)
    ! if (lai_grass(gn) > lai_opt) then
    !    !a1: excess gmass_leaf (g drymatter/m2)
    !    a1             = (lai_grass(gn)-lai_opt) / SLA(p)
    !    !a2: gmass_leaf that senescence (g drymatter/m2)
    !    a2             = a1 * 0.020
    !    gmass_leaf(gn) = gmass_leaf(gn) - a2
    !    pool_litter    = pool_litter    + a2 * area_cover
    !    lai_grass(gn)  = gmass_leaf(gn) * SLA(p)
    ! end if
    
    !root mass supplement
       !a1: required root mass (g dm / m2)
       a1 = max(0.0, gmass_leaf(gn)/FR_ratio(p) - gmass_root(gn))
       !a2: maximum root mass increament (g dm / m2)
       a2 = gmass_available(gn) / RG_r(p)
       !a3: actual increamet of root mass (g dm / m2)
       a3 = min(a1, a2)
       !root mass supplement procedure
       gmass_root(gn)      = gmass_root(gn)      + a3          
       gmass_available(gn) = gmass_available(gn) - a3 * RG_r(p)
       mass_combust        = mass_combust + (a3 * RG_r(p) - a3) * area_cover
       npp(p)              = npp(p)       - (a3 * RG_r(p) - a3) * area_cover
    
    !stock resource supplement
    if ( counter > pheno_clo(p)+30 ) then !after more than 30 days from foliation
       !a1:resource requirement for stock (g/m2)
        a1 = max( 0.0, gmass_leaf(gn) - gmass_stock(gn) )
       !a2:maximum increment of mass_stock with resource available (g/m2)
        a2 = gmass_available(gn) / RG_stock_in
       !a3:actual increment of mass_stock (g/m2)
        a3 = min( a1, a2 )
       !stock mass increment precedure
       gmass_stock(gn)    = gmass_stock(gn)     +  a3               
       gmass_available(gn)= gmass_available(gn) -  a3 * RG_stock_in 
       mass_combust       = mass_combust + (a3 * RG_stock_in - a3) * area_cover
       npp(p)             = npp(p)       - (a3 * RG_stock_in - a3) * area_cover
    endif
    
    !foliage growth
       !a1: maximum foliage mass requirement (g/m2)
        a1 = (lai_opt - lai_grass(gn)) / SLA(p)
        a1 = max(a1, 0.0)
       !a2: maximum increament of folige mass that available resource permits (g/m2)
        a2 = gmass_available(gn) / RG_f(p)
       !a3: actual increament of foliage mass (g/m2)
        a3 = min(a1, a2)
       !foliage mass increament procedure
        gmass_leaf(gn)      = gmass_leaf(gn)      +  a3          
        gmass_available(gn) = gmass_available(gn) -  a3 * RG_f(p)
        mass_combust        = mass_combust   + (a3 * RG_f(p) - a3) * area_cover
        npp(p)              = npp(p)         - (a3 * RG_f(p) - a3) * area_cover
        lai_grass(gn)       = gmass_leaf(gn) * SLA(p)
    
    !reproduction: excess resource will be added to litter
    if ( counter > pheno_clo(p)+30    ) then !after more than 30 days from foliation
    if ( gmass_stock(gn) > 100.0 ) then !minimum stock resource for reproduction (g dm /m2)
       pool_litter         = pool_litter + gmass_available(gn)*area_cover
       gmass_available(gn) = 0.0000
    endif
    endif
    
END IF
END DO

!carbon emission
   c_emission = c_emission + mass_combust * C_in_drymass

!carbon leak checker (after procedure)
   a1 = gmass_leaf(1)+gmass_root(1)+gmass_available(1)+gmass_stock(1)
   a2 = gmass_leaf(2)+gmass_root(2)+gmass_available(2)+gmass_stock(2)
   
   if (type_grass==3) then
      a3 = (0.9*a1 + 0.1*a2)*Max_loc*Max_loc + pool_litter + mass_combust
   else
      a3 = (0.1*a1 + 0.9*a2)*Max_loc*Max_loc + pool_litter + mass_combust
   end if
   a3 = abs(a3-mass_total)
   
   if ( a3>1.0 .and. Logging==1 ) then
      write (*,*) 'Error @ growth grass: leaking mass (g/m2) ->', a3, '   doy ->', doy
   end if
   
End subroutine growth_grass



!**************************************************************************************************
! Crown depth adjustment
!**************************************************************************************************
SUBROUTINE crown_adjust &
(height, pft, tree_exist, bole, la, mass_leaf, pool_litter, stat_leaf_crownbottom, stat_leaf_crowntop)

USE data_structure
implicit none

!Define Arguments
integer,dimension(1:Max_no),intent(IN)   ::height     !tree height from 1.3m above ground (STEP)
integer,dimension(1:Max_no),intent(IN)   ::pft        !Plant Functional Type (1-PFT_no)
logical,dimension(1:Max_no),intent(IN)   ::tree_exist !tree exist or not (.TRUE. or .FALSE.)

integer,dimension(1:Max_no),intent(INOUT)::bole        !bole length (STEP)
real   ,dimension(1:Max_no),intent(INOUT)::la          !leaf area (m2)
real   ,dimension(1:Max_no),intent(INOUT)::mass_leaf      !biomass of foliage (g)
real                       ,intent(INOUT)::pool_litter !litter pool in a stand (g/stand)

real   ,dimension(1:Max_no, 10),intent(INOUT)::stat_leaf_crownbottom !annual NPP of foliage bottom
real   ,dimension(1:Max_no    ),intent(INOUT)::stat_leaf_crowntop    !annual NPP of foliage top


!Define local variables
integer Crown_depth_min !minimum crown depth that can not be devoted for adjustment (STEP)
real    a1, a2     !for general usage
integer no,i       !for general usage
real    mass_total !mass before procedure --for checker-- (g)
real    la_step    !Leaf area for each foliage layer (m2/STEP)
integer count

!Save mass_total for carbon-leak-checker
mass_total = sum(mass_leaf) + pool_litter

Crown_depth_min = 10

!Bole-lengh adjustment
DO no=1, Max_no 
if ( .not. tree_exist(no) )       cycle
if ( height(no)-bole(no) <= Crown_depth_min ) cycle !at least <=2
   
    !determine number of crown disks that will be changed to bole
    count = 0
    do i = 1, min(10, height(no)-bole(no)-1)
       if ( sum(stat_leaf_crownbottom(no,1:i))/ i  < &
            stat_leaf_crowntop(no) * ALM4(pft(no)) ) count = i
    end do
    count = min(count, height(no)-bole(no)-Crown_depth_min)
    
    !when maximum_crown_depth was assumed
    ! max_depth = 30
    ! if ( height(no)-bole(no)-max_depth > 0 ) then
    !    count = max(count, height(no)-bole(no)-max_depth )
    ! end if
    
    !crown adjust procedures
    if (count > 0) then
    !  la_step     = la(no) / real( height(no)-bole(no) )
    !  la(no)      = la(no)      - la_step * count
    !  mass_leaf(no)  = mass_leaf(no)  - la_step * count / SLA(pft(no))
    !  pool_litter = pool_litter + la_step * count / SLA(pft(no))
       bole(no)    = bole(no)    + count
    endif
   
END DO

!reset bole-height-adjust control variables
stat_leaf_crownbottom(:,:) = 0.0
stat_leaf_crowntop(:)      = 0.0

!carbon leak checker
a1 = sum(mass_leaf) + pool_litter !total carbon
a2 = abs(a1 - mass_total)      !difference in total carbon
if ( a2>5.0 .and. Logging==1 ) then
     write (*,*) 'Error @ crown_adjust: leaking mass (g) ->', a2
end if

END SUBROUTINE crown_adjust



!**************************************************************************************************
! Monthly growth and reproduction of woody PFT (each individual_tree procedure)
!**************************************************************************************************
SUBROUTINE growth_stem &
    (counter, pheno_clo, phenology, &
    pool_litter, c_emission, &
    height_limit, radius_limit, pft, tree_exist, &
    height, dbh_heartwood, dbh_sapwood, crown_diameter, crown_area, &
    mass_leaf, mass_trunk, mass_root, mass_stock, mass_available,&
    npp, mort_regu1 )

USE data_structure
implicit none

!Define arguments
integer                  ,intent(IN)::counter    !simulation day (1-Simulation_year*365)
integer,dimension(PFT_no),intent(IN)::pheno_clo  !counter number of last leaf onset
logical,dimension(PFT_no),intent(IN)::phenology  !phenological status (T:foliation, F:dormance)

real              ,intent(INOUT)::pool_litter  !litter pool in a stand (g / stand)
real              ,intent(INOUT)::c_emission   !monthly carbon emission (C g / stand)

integer,dimension(1:Max_no),intent(IN)   ::height_limit   !max tree height in this point
real   ,dimension(1:Max_no),intent(IN)   ::radius_limit   !max canopy radius in this point
integer,dimension(1:Max_no),intent(IN)   ::pft            !Plant Functional Type (1-PFT_no)
logical,dimension(1:Max_no),intent(IN)   ::tree_exist     !tree exist or not (.TRUE. or .FALSE.)
integer,dimension(1:Max_no),intent(INOUT)::height         !tree over 1.3m above ground (STEP)
real   ,dimension(1:Max_no),intent(INOUT)::dbh_heartwood  !heartwood diameter at 1.3m (m)
real   ,dimension(1:Max_no),intent(INOUT)::dbh_sapwood    !sapwood   diameter at 1.3m (m)
real   ,dimension(1:Max_no),intent(INOUT)::crown_diameter !crown diameter (m)
real   ,dimension(1:Max_no),intent(INOUT)::crown_area     !crown area (m2)

real   ,dimension(1:Max_no),intent(IN)   ::mass_leaf      !biomass of foliage (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_trunk     !biomass of stem    (g)
real   ,dimension(1:Max_no),intent(IN)   ::mass_root      !biomass of root    (g)
real   ,dimension(1:Max_no),intent(IN)   ::mass_stock     !biomass for stock  (g)
real   ,dimension(1:Max_no),intent(INOUT)::mass_available !biomass available  (g)

real,dimension(PFT_no)  ,intent(INOUT)::npp        !NPP for each PFT (g dm / stand / yr)
real,dimension(1:Max_no),intent(INOUT)::mort_regu1 !sum of NPP within the previous year (g dm)

!Define local variables
real    ca_limit     !crown area limit
real    diameter_new !new diameter (m)
integer height_new   !new tree height (STEP)
real    mass_combust !biomass combust (g DM/day/stand)
real    mass_total   !total biomass before procedure --for checker-- (g DM/stand)
real    dn, md, up   !for convergence loop
real    a1, a2, x, y !for general usage
integer no, p, count !

!for checker
mass_combust = 0.000
mass_total   = sum(mass_trunk) + sum(mass_available) + pool_litter + mass_combust

!*** EACH TREE LOOP in ***
DO no=1, Max_no 

p = pft(no) !set pft number

IF ( tree_exist(no) )            then !when tree exist
IF ( phenology(pft(no)) )        then !when on foliation phase
IF ( counter > pheno_clo(p)+21 ) then !after more than 21 days from foliation
IF ( mass_available(no) > 0.0 )  then !when mass available exist
   
   !Reproduction
   x = mass_leaf(no) + mass_trunk(no) + mass_root(no) + mass_stock(no) + mass_available(no)
   y = 0.10             !fraction of available resource for reproduction (0.0-1.0)
   If (x > 1000.0) then !reproduce only when total biomass is more than 1kg
      pool_litter        = pool_litter + mass_available(no) * y
      mass_available(no) = mass_available(no) * (1.0-y)
   Endif
   
   !Allocation to sapwood
   x                  = mass_available(no) / RG_s(p)
   mass_trunk(no)     = mass_trunk(no)     +  x
   mass_combust       = mass_combust   +  mass_available(no) - x
   npp(p)             = npp(p)         - (mass_available(no) - x)
   mort_regu1(no)     = mort_regu1(no) - (mass_available(no) - x)
   mass_available(no) = 0.0
   
   !determine DBH increament
   up = 0.5000000; dn = 0.0000000 !lengh of examine convergence
   count = 0
   Do
      count = count + 1
      !current sapwood diameter increament (m)
      md           = (up+dn) / 2.0
      !current diameter (m)
      diameter_new = dbh_heartwood(no) + dbh_sapwood(no) + md
      !current height (STEP)
      height_new   = max(height(no), min(height_limit(no), h(diameter_new, p) ))
      !a1: current stem mass (g)
      a1 = ALM3(p) * ( (diameter_new**2)*PI/4.0 ) * (1.3+height_new*STEP)
      !a2: difference in stem mass (g)
      a2 = a1 - mass_trunk(no)
      !examine convergence
      if    (a2 < 0.0000000) then ; dn = md
      else                        ; up = md
      end if                               
      if (up-dn < 0.00001) exit !error should be less than 1/100 mm/month
      if (count > 2000)    exit !avoid infinite loop
   End do
   
   !update trunc diameter and height
   height(no)         = height_new
   dbh_sapwood(no)    = dbh_sapwood(no) + md
   
   !adjust crown diameter
   ca_limit           = PI * radius_limit(no) * radius_limit(no)
   crown_area(no)     = max(0.001, min(ca_limit, ca(diameter_new, p)) )
   crown_diameter(no) = 2 * sqrt(crown_area(no)/PI)
   
END IF
END IF
END IF
END IF
END DO

!carbon leak checker
a1 = sum(mass_trunk) + sum(mass_available) + pool_litter + mass_combust
a2 = abs(mass_total - a1)
if ( a2>100.0 .and. Logging==1 ) then
   write (*,*) 'Error @ growth_stem: leaking mass (g) ->', a2
end if

!carbon emission
c_emission = c_emission + mass_combust * C_in_drymass

END SUBROUTINE growth_stem



!**************************************************************************************************
! Calculation of photosynthesis parameters (for each PFT)
!**************************************************************************************************
SUBROUTINE decomposition &
         (pool_litter, pool_som_int, pool_som_slow, c_emission, &
         aet, tmp_soil_month_ave, pool_w1, pool_w2, W_sat)

USE data_structure
implicit none

!*** Define common arguments ***

!soil carbon
real,intent(INOUT)::pool_litter   !litter pool in a stand (g dm / stand)
real,intent(INOUT)::pool_som_int  !intermediated SOM pool (g dm / stand)
real,intent(INOUT)::pool_som_slow !slow SOM pool          (g dm / stand)

!carbon flux recorder
real,intent(INOUT)::c_emission    !monthly carbon emission (g C / month / stand)

!climate and soilmoisture
real,intent(IN)   ::aet                !sum of actual evapotranspiration of last year (mm/year)
real,intent(IN)   ::tmp_soil_month_ave !30days running average of tmp_soil (Celcius)
real,intent(IN)   ::pool_w1            !soil water content of soil layer 1 (mm)
real,intent(IN)   ::pool_w2            !soil water content of soil layer 2 (mm)
real,intent(IN)   ::W_sat              !saturate point   (m3/m3, 0.0 -> 1.0)

!*** Define local variable ***
real effect_tmp      !temperature coefficient for decomposition
real effect_mois     !moisture    coefficient for decomposition

real d_rate_litter   !decomposition rate of litter   (month-1)
real d_rate_som_int  !decomposition rate of som_int  (month-1)
real d_rate_som_slow !decomposition rate of som_slow (month-1)

real mass_decomp     !decomposed organic matter (g dm month-1 stand-1)

real x               !for general usage

real c_before        !total carbon before procedure   -for checker- (g C/stand)
real c_after         !total carbon before procedure   -for checker- (g C/stand)

!Checker
   c_before = (pool_litter+pool_som_int+pool_som_slow)*C_in_drymass + c_emission
   
!Calculatet decomposition coefficients
  !effect_tmp     = exp( 0.230259 * (tmp_soil_month_ave-20.0) ) !Q10 method
   effect_tmp     = exp( 308.56 * ( 1.0/66.02 - 1.0/( max(-45.0, tmp_soil_month_ave)+46.02) ) )
   effect_mois    = 0.25 + 0.75 * (pool_w1 / W_sat / Depth1)
   
!Calculate decomposition rates
   !d_rate_litter  =  effect_tmp * effect_mois * (Turnover_litter/12.0)
   d_rate_litter   = ( 10**(0.0014175*aet-1.4553) ) / 12.0
   d_rate_som_int  = (effect_tmp * effect_mois) * (TO_fast  / 12.0)
   d_rate_som_slow = (effect_tmp * effect_mois) * (TO_slow / 12.0)
   
   d_rate_litter   = min(1.0, d_rate_litter  )
   d_rate_som_int  = min(1.0, d_rate_som_int )
   d_rate_som_slow = min(1.0, d_rate_som_slow)

!decomposition procedure1, litter
   mass_decomp     = pool_litter * d_rate_litter
   !write (*,*) '1 ->', pool_litter, d_rate_litter, mass_decomp
   pool_litter     = pool_litter   - mass_decomp
   pool_som_int    = pool_som_int  + mass_decomp * (1.00-F_air) * F_inter
   pool_som_slow   = pool_som_slow + mass_decomp * (1.00-F_air) * (1.00 - F_inter)
   c_emission      = c_emission    + mass_decomp * F_air * C_in_drymass
   
!decomposition procedure2, s.o.m. with intermediated composition rate
   mass_decomp     = pool_som_int * d_rate_som_int
   pool_som_int    = pool_som_int - mass_decomp
   c_emission      = c_emission   + mass_decomp * C_in_drymass
   
!decomposition procedure3, s.o.m. with slow composition rate
   mass_decomp     = pool_som_slow * d_rate_som_slow
   pool_som_slow   = pool_som_slow - mass_decomp
   c_emission      = c_emission    + mass_decomp * C_in_drymass
   
!carbon leak checker
   c_after = (pool_litter+pool_som_int+pool_som_slow)*C_in_drymass + c_emission
   x = c_before - c_after
   
   if ( abs(x)>10.0 .and. Logging==1 ) then
      write (*,*) 'Error @ decomposiotion: leaking mass (g) ->', x
   end if
   
END SUBROUTINE decomposition